@@ -133,7 +133,7 @@ impl WasmMmap {
 
     /// Copies the memory
     /// (in this case it performs a copy-on-write to save memory)
-    pub fn duplicate(&mut self) -> Result<Self, MemoryError> {
+    pub fn copy(&mut self) -> Result<Self, MemoryError> {
         let mem_length = self.size.bytes().0;
         let mut alloc = self
             .alloc
@@ -304,9 +304,9 @@ impl VMOwnedMemory {
     }
 
     /// Copies this memory to a new memory
-    pub fn duplicate(&mut self) -> Result<Self, MemoryError> {
+    pub fn copy(&mut self) -> Result<Self, MemoryError> {
         Ok(Self {
-            mmap: self.mmap.duplicate()?,
+            mmap: self.mmap.copy()?,
             config: self.config.clone(),
         })
     }
@@ -343,14 +343,13 @@ impl LinearMemory for VMOwnedMemory {
     }
 
     /// Owned memory can not be cloned (this will always return None)
-    fn try_clone(&self) -> Option<Box<dyn LinearMemory + 'static>> {
-        tracing::warn!("trying to clone owned memory");
-        None
+    fn try_clone(&self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        Err(MemoryError::MemoryNotShared)
     }
 
     /// Copies this memory to a new memory
-    fn duplicate(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
-        let forked = Self::duplicate(self)?;
+    fn copy(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        let forked = Self::copy(self)?;
         Ok(Box::new(forked))
     }
 }
@@ -393,10 +392,10 @@ impl VMSharedMemory {
     }
 
     /// Copies this memory to a new memory
-    pub fn duplicate(&mut self) -> Result<Self, MemoryError> {
+    pub fn copy(&mut self) -> Result<Self, MemoryError> {
         let mut guard = self.mmap.write().unwrap();
         Ok(Self {
-            mmap: Arc::new(RwLock::new(guard.duplicate()?)),
+            mmap: Arc::new(RwLock::new(guard.copy()?)),
             config: self.config.clone(),
             conditions: ThreadConditions::new(),
         })
@@ -440,13 +439,13 @@ impl LinearMemory for VMSharedMemory {
     }
 
     /// Shared memory can always be cloned
-    fn try_clone(&self) -> Option<Box<dyn LinearMemory + 'static>> {
-        Some(Box::new(self.clone()))
+    fn try_clone(&self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        Ok(Box::new(self.clone()))
     }
 
     /// Copies this memory to a new memory
-    fn duplicate(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
-        let forked = Self::duplicate(self)?;
+    fn copy(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        let forked = Self::copy(self)?;
         Ok(Box::new(forked))
     }
 
@@ -515,7 +514,7 @@ impl LinearMemory for VMMemory {
     }
 
     /// Attempts to clone this memory (if its clonable)
-    fn try_clone(&self) -> Option<Box<dyn LinearMemory + 'static>> {
+    fn try_clone(&self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
         self.0.try_clone()
     }
 
@@ -525,8 +524,8 @@ impl LinearMemory for VMMemory {
     }
 
     /// Copies this memory to a new memory
-    fn duplicate(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
-        self.0.duplicate()
+    fn copy(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        self.0.copy()
     }
 }
 
@@ -588,8 +587,8 @@ impl VMMemory {
     }
 
     /// Copies this memory to a new memory
-    pub fn duplicate(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
-        LinearMemory::duplicate(self)
+    pub fn copy(&mut self) -> Result<Box<dyn LinearMemory + 'static>, MemoryError> {
+        LinearMemory::copy(self)
     }
 }
 
