@@ -1,4 +1,5 @@
 use std::collections::HashMap;
+use std::collections::VecDeque;
 use std::future::Future;
 use std::net::IpAddr;
 use std::net::SocketAddr;
@@ -87,6 +88,7 @@ impl RemoteNetworkingClient {
             recv_tx: Default::default(),
             recv_with_addr_tx: Default::default(),
             accept_tx: Default::default(),
+            sent_tx: Default::default(),
             handlers: Default::default(),
             stall: Default::default(),
         };
@@ -257,6 +259,9 @@ impl RemoteNetworkingClient {
         let (tx, rx_accept) = tokio::sync::mpsc::channel(100);
         self.common.accept_tx.lock().unwrap().insert(id, tx);
 
+        let (tx, rx_sent) = tokio::sync::mpsc::channel(100);
+        self.common.sent_tx.lock().unwrap().insert(id, tx);
+
         RemoteSocket {
             socket_id: id,
             common: self.common.clone(),
@@ -264,8 +269,12 @@ impl RemoteNetworkingClient {
             rx_recv,
             rx_recv_with_addr,
             rx_accept,
+            rx_sent,
             tx_waker: TxWaker::new(&self.common).as_waker(),
             pending_accept: None,
+            buffer_accept: Default::default(),
+            buffer_recv_with_addr: Default::default(),
+            send_available: 0,
         }
     }
 }
@@ -335,7 +344,7 @@ impl Future for RemoteNetworkingClientDriver {
 
                                 if let Some(h) = common.handlers.lock().unwrap().get_mut(&socket_id)
                                 {
-                                    h.interest(InterestType::Readable)
+                                    h.push_interest(InterestType::Readable)
                                 }
                             }));
                         }
@@ -357,15 +366,27 @@ impl Future for RemoteNetworkingClientDriver {
 
                                 if let Some(h) = common.handlers.lock().unwrap().get_mut(&socket_id)
                                 {
-                                    h.interest(InterestType::Readable)
+                                    h.push_interest(InterestType::Readable)
                                 }
                             }));
                         }
-                        MessageResponse::Sent { socket_id, .. } => {
+                        MessageResponse::Sent {
+                            socket_id, amount, ..
+                        } => {
+                            let tx = {
+                                let guard = self.common.sent_tx.lock().unwrap();
+                                match guard.get(&socket_id) {
+                                    Some(tx) => tx.clone(),
+                                    None => continue,
+                                }
+                            };
+                            self.tasks.push_back(Box::pin(async move {
+                                tx.send(amount).await.ok();
+                            }));
                             if let Some(h) =
                                 self.common.handlers.lock().unwrap().get_mut(&socket_id)
                             {
-                                h.interest(InterestType::Writable)
+                                h.push_interest(InterestType::Writable)
                             }
                         }
                         MessageResponse::SendError {
@@ -377,14 +398,14 @@ impl Future for RemoteNetworkingClientDriver {
                                 if let Some(h) =
                                     self.common.handlers.lock().unwrap().get_mut(&socket_id)
                                 {
-                                    h.interest(InterestType::Closed)
+                                    h.push_interest(InterestType::Closed)
                                 }
                             }
                             _ => {
                                 if let Some(h) =
                                     self.common.handlers.lock().unwrap().get_mut(&socket_id)
                                 {
-                                    h.interest(InterestType::Writable)
+                                    h.push_interest(InterestType::Writable)
                                 }
                             }
                         },
@@ -408,7 +429,7 @@ impl Future for RemoteNetworkingClientDriver {
 
                                 if let Some(h) = common.handlers.lock().unwrap().get_mut(&socket_id)
                                 {
-                                    h.interest(InterestType::Readable)
+                                    h.push_interest(InterestType::Readable)
                                 }
                             }));
                         }
@@ -416,7 +437,7 @@ impl Future for RemoteNetworkingClientDriver {
                             if let Some(h) =
                                 self.common.handlers.lock().unwrap().get_mut(&socket_id)
                             {
-                                h.interest(InterestType::Closed)
+                                h.push_interest(InterestType::Closed)
                             }
                         }
                         MessageResponse::ResponseToRequest { req_id, res } => {
@@ -449,7 +470,7 @@ impl TxWaker {
     fn wake_now(&self) {
         let mut guard = self.common.handlers.lock().unwrap();
         for (_, handler) in guard.iter_mut() {
-            handler.interest(InterestType::Writable);
+            handler.push_interest(InterestType::Writable);
         }
     }
 
@@ -519,6 +540,7 @@ struct RemoteCommon {
     recv_tx: Mutex<SocketMap<mpsc::Sender<Vec<u8>>>>,
     recv_with_addr_tx: Mutex<SocketMap<mpsc::Sender<DataWithAddr>>>,
     accept_tx: Mutex<SocketMap<mpsc::Sender<SocketWithAddr>>>,
+    sent_tx: Mutex<SocketMap<mpsc::Sender<u64>>>,
     #[derivative(Debug = "ignore")]
     handlers: Mutex<SocketMap<Box<dyn virtual_mio::InterestHandler + Send + Sync>>>,
 
@@ -849,7 +871,11 @@ struct RemoteSocket {
     rx_recv_with_addr: mpsc::Receiver<DataWithAddr>,
     tx_waker: Waker,
     rx_accept: mpsc::Receiver<SocketWithAddr>,
+    rx_sent: mpsc::Receiver<u64>,
     pending_accept: Option<SocketId>,
+    buffer_recv_with_addr: VecDeque<DataWithAddr>,
+    buffer_accept: VecDeque<SocketWithAddr>,
+    send_available: u64,
 }
 impl Drop for RemoteSocket {
     fn drop(&mut self) {
@@ -913,6 +939,57 @@ impl VirtualIoSource for RemoteSocket {
     fn remove_handler(&mut self) {
         self.common.handlers.lock().unwrap().remove(&self.socket_id);
     }
+
+    fn poll_read_ready(&mut self, cx: &mut Context<'_>) -> Poll<Result<usize>> {
+        if !self.rx_buffer.is_empty() {
+            return Poll::Ready(Ok(self.rx_buffer.len()));
+        }
+        match self.rx_recv.poll_recv(cx) {
+            Poll::Ready(Some(data)) => {
+                self.rx_buffer.extend_from_slice(&data);
+                return Poll::Ready(Ok(self.rx_buffer.len()));
+            }
+            Poll::Ready(None) => return Poll::Ready(Ok(0)),
+            Poll::Pending => {}
+        }
+        if !self.buffer_recv_with_addr.is_empty() {
+            let total = self
+                .buffer_recv_with_addr
+                .iter()
+                .map(|a| a.data.len())
+                .sum();
+            return Poll::Ready(Ok(total));
+        }
+        match self.rx_recv_with_addr.poll_recv(cx) {
+            Poll::Ready(Some(data)) => self.buffer_recv_with_addr.push_back(data),
+            Poll::Ready(None) => return Poll::Ready(Ok(0)),
+            Poll::Pending => {}
+        }
+        if !self.buffer_accept.is_empty() {
+            return Poll::Ready(Ok(self.buffer_accept.len()));
+        }
+        match self.rx_accept.poll_recv(cx) {
+            Poll::Ready(Some(data)) => self.buffer_accept.push_back(data),
+            Poll::Ready(None) => {}
+            Poll::Pending => {}
+        }
+        Poll::Pending
+    }
+
+    fn poll_write_ready(&mut self, cx: &mut Context<'_>) -> Poll<Result<usize>> {
+        if self.send_available > 0 {
+            return Poll::Ready(Ok(self.send_available as usize));
+        }
+        match self.rx_sent.poll_recv(cx) {
+            Poll::Ready(Some(amt)) => {
+                self.send_available += amt;
+                return Poll::Ready(Ok(self.send_available as usize));
+            }
+            Poll::Ready(None) => return Poll::Ready(Ok(0)),
+            Poll::Pending => {}
+        }
+        Poll::Pending
+    }
 }
 
 impl VirtualSocket for RemoteSocket {
@@ -968,12 +1045,16 @@ impl VirtualSocket for RemoteSocket {
 
 impl VirtualTcpListener for RemoteSocket {
     fn try_accept(&mut self) -> Result<(Box<dyn VirtualTcpSocket + Sync>, SocketAddr)> {
+        // We may already have accepted a connection in the `poll_read_ready` method
         self.touch_begin_accept()?;
-
-        let accepted = self.rx_accept.try_recv().map_err(|err| match err {
-            TryRecvError::Empty => NetworkError::WouldBlock,
-            TryRecvError::Disconnected => NetworkError::ConnectionAborted,
-        })?;
+        let accepted = if let Some(child) = self.buffer_accept.pop_front() {
+            child
+        } else {
+            self.rx_accept.try_recv().map_err(|err| match err {
+                TryRecvError::Empty => NetworkError::WouldBlock,
+                TryRecvError::Disconnected => NetworkError::ConnectionAborted,
+            })?
+        };
 
         // This placed here will mean there is always an accept request pending at the
         // server as the constructor invokes this method and we invoke it here after
@@ -1003,6 +1084,13 @@ impl VirtualTcpListener for RemoteSocket {
             .unwrap()
             .insert(accepted.socket, tx);
 
+        let (tx, rx_sent) = tokio::sync::mpsc::channel(100);
+        self.common
+            .sent_tx
+            .lock()
+            .unwrap()
+            .insert(accepted.socket, tx);
+
         let socket = RemoteSocket {
             socket_id: accepted.socket,
             common: self.common.clone(),
@@ -1010,8 +1098,12 @@ impl VirtualTcpListener for RemoteSocket {
             rx_recv,
             rx_recv_with_addr,
             rx_accept,
+            rx_sent,
             pending_accept: None,
             tx_waker: TxWaker::new(&self.common).as_waker(),
+            buffer_accept: Default::default(),
+            buffer_recv_with_addr: Default::default(),
+            send_available: 0,
         };
         Ok((Box::new(socket), accepted.addr))
     }
@@ -1062,8 +1154,11 @@ impl VirtualRawSocket for RemoteSocket {
             },
         ) {
             Poll::Ready(Ok(())) => Ok(data.len()),
+            Poll::Ready(Err(NetworkError::WouldBlock)) | Poll::Pending => {
+                self.send_available = 0;
+                Err(NetworkError::WouldBlock)
+            }
             Poll::Ready(Err(err)) => Err(err),
-            Poll::Pending => Err(NetworkError::WouldBlock),
         }
     }
 
@@ -1078,8 +1173,11 @@ impl VirtualRawSocket for RemoteSocket {
             },
         ) {
             Poll::Ready(Ok(())) => Ok(()),
+            Poll::Ready(Err(NetworkError::WouldBlock)) | Poll::Pending => {
+                self.send_available = 0;
+                Err(NetworkError::WouldBlock)
+            }
             Poll::Ready(Err(err)) => Err(err),
-            Poll::Pending => Err(NetworkError::WouldBlock),
         }
     }
 
@@ -1130,8 +1228,11 @@ impl VirtualConnectionlessSocket for RemoteSocket {
             },
         ) {
             Poll::Ready(Ok(())) => Ok(data.len()),
+            Poll::Ready(Err(NetworkError::WouldBlock)) | Poll::Pending => {
+                self.send_available = 0;
+                Err(NetworkError::WouldBlock)
+            }
             Poll::Ready(Err(err)) => Err(err),
-            Poll::Pending => Err(NetworkError::WouldBlock),
         }
     }
 
@@ -1370,6 +1471,36 @@ impl VirtualTcpSocket for RemoteSocket {
         }
     }
 
+    fn set_keepalive(&mut self, keep_alive: bool) -> Result<()> {
+        self.io_socket_fire_and_forget(RequestType::SetKeepAlive(keep_alive))
+    }
+
+    fn keepalive(&self) -> Result<bool> {
+        match InlineWaker::block_on(self.io_socket(RequestType::GetKeepAlive)) {
+            ResponseType::Err(err) => Err(err),
+            ResponseType::Flag(val) => Ok(val),
+            res => {
+                tracing::debug!("invalid response to get nodelay request - {res:?}");
+                Err(NetworkError::IOError)
+            }
+        }
+    }
+
+    fn set_dontroute(&mut self, dont_route: bool) -> Result<()> {
+        self.io_socket_fire_and_forget(RequestType::SetDontRoute(dont_route))
+    }
+
+    fn dontroute(&self) -> Result<bool> {
+        match InlineWaker::block_on(self.io_socket(RequestType::GetDontRoute)) {
+            ResponseType::Err(err) => Err(err),
+            ResponseType::Flag(val) => Ok(val),
+            res => {
+                tracing::debug!("invalid response to get nodelay request - {res:?}");
+                Err(NetworkError::IOError)
+            }
+        }
+    }
+
     fn addr_peer(&self) -> Result<SocketAddr> {
         match InlineWaker::block_on(self.io_socket(RequestType::GetAddrPeer)) {
             ResponseType::Err(err) => Err(err),
