@@ -71,19 +71,20 @@ async fn setup_pipe(
 
 #[cfg(feature = "remote")]
 async fn test_tcp(client: RemoteNetworkingClient, _server: RemoteNetworkingServer) {
-    static PORT: AtomicU16 = AtomicU16::new(8000);
-    let addr = SocketAddr::V4(SocketAddrV4::new(
-        Ipv4Addr::LOCALHOST,
-        PORT.fetch_add(1, Ordering::SeqCst),
-    ));
-    tracing::info!("listening on {addr}");
     let mut listener = client
-        .listen_tcp(addr.clone(), false, false, false)
+        .listen_tcp(
+            SocketAddr::from((Ipv4Addr::LOCALHOST, 0)),
+            false,
+            false,
+            false,
+        )
         .await
         .unwrap();
+    let addr = listener.addr_local().unwrap();
+    tracing::info!("listening on {addr}");
 
-    const TEST1: &'static str = "the cat ran up the wall!";
-    const TEST2: &'static str = "...and fell off the roof! raise the roof! oop oop";
+    const TEST1: &str = "the cat ran up the wall!";
+    const TEST2: &str = "...and fell off the roof! raise the roof! oop oop";
 
     tracing::info!("spawning acceptor worker thread");
     tokio::task::spawn(async move {
@@ -104,10 +105,7 @@ async fn test_tcp(client: RemoteNetworkingClient, _server: RemoteNetworkingServe
 
     tracing::info!("connecting to listening socket");
     let mut socket = client
-        .connect_tcp(
-            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0)),
-            addr,
-        )
+        .connect_tcp(SocketAddr::from((Ipv4Addr::LOCALHOST, 0)), addr)
         .await
         .unwrap();
 
@@ -125,27 +123,27 @@ async fn test_tcp(client: RemoteNetworkingClient, _server: RemoteNetworkingServe
 }
 
 #[cfg(feature = "remote")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_mpsc() {
     let (client, server) = setup_mpsc().await;
     test_tcp(client, server).await
 }
 
 #[cfg(feature = "remote")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_small_pipe_using_bincode() {
     let (client, server) = setup_pipe(10, FrameSerializationFormat::Bincode).await;
     test_tcp(client, server).await
 }
 
 #[cfg(feature = "remote")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_large_pipe_using_bincode() {
     let (client, server) = setup_pipe(1024000, FrameSerializationFormat::Bincode).await;
     test_tcp(client, server).await
@@ -153,9 +151,9 @@ async fn test_tcp_with_large_pipe_using_bincode() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "json")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_small_pipe_using_json() {
     let (client, server) = setup_pipe(10, FrameSerializationFormat::Json).await;
     test_tcp(client, server).await
@@ -163,9 +161,9 @@ async fn test_tcp_with_small_pipe_using_json() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "json")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_large_pipe_json_using_json() {
     let (client, server) = setup_pipe(1024000, FrameSerializationFormat::Json).await;
     test_tcp(client, server).await
@@ -173,9 +171,9 @@ async fn test_tcp_with_large_pipe_json_using_json() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "messagepack")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_small_pipe_using_messagepack() {
     let (client, server) = setup_pipe(10, FrameSerializationFormat::MessagePack).await;
     test_tcp(client, server).await
@@ -183,9 +181,9 @@ async fn test_tcp_with_small_pipe_using_messagepack() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "messagepack")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_large_pipe_json_using_messagepack() {
     let (client, server) = setup_pipe(1024000, FrameSerializationFormat::MessagePack).await;
     test_tcp(client, server).await
@@ -193,9 +191,9 @@ async fn test_tcp_with_large_pipe_json_using_messagepack() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "cbor")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_small_pipe_using_cbor() {
     let (client, server) = setup_pipe(10, FrameSerializationFormat::Cbor).await;
     test_tcp(client, server).await
@@ -203,15 +201,15 @@ async fn test_tcp_with_small_pipe_using_cbor() {
 
 #[cfg(feature = "remote")]
 #[cfg(feature = "cbor")]
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn test_tcp_with_large_pipe_json_using_cbor() {
     let (client, server) = setup_pipe(1024000, FrameSerializationFormat::Cbor).await;
     test_tcp(client, server).await
 }
 
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
 #[tokio::test]
 async fn test_google_poll() {
@@ -302,7 +300,7 @@ async fn test_google_poll() {
     tracing::info!("done");
 }
 
-#[cfg(target_os = "linux")]
+#[cfg_attr(windows, ignore)]
 #[traced_test]
 #[tokio::test]
 async fn test_google_epoll() {
