@@ -20,7 +20,7 @@ enum JoinStatusResult {
 /// ## Parameters
 ///
 /// * `pid` - Handle of the child process to wait on
-//#[instrument(level = "trace", skip_all, fields(pid = ctx.data().process.pid().raw()), ret, err)]
+//#[instrument(level = "trace", skip_all, fields(pid = ctx.data().process.pid().raw()), ret)]
 pub fn proc_join<M: MemorySize + 'static>(
     ctx: FunctionEnvMut<'_, WasiEnv>,
     pid_ptr: WasmPtr<OptionPid, M>,
@@ -33,7 +33,7 @@ pub fn proc_join<M: MemorySize + 'static>(
 pub(super) fn proc_join_internal<M: MemorySize + 'static>(
     mut ctx: FunctionEnvMut<'_, WasiEnv>,
     pid_ptr: WasmPtr<OptionPid, M>,
-    _flags: JoinFlags,
+    flags: JoinFlags,
     status_ptr: WasmPtr<JoinStatus, M>,
 ) -> Result<Errno, WasiError> {
     wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
@@ -182,20 +182,32 @@ pub(super) fn proc_join_internal<M: MemorySize + 'static>(
             }
         ));
 
-        // Wait for the process to finish
-        let process2 = process.clone();
-        let res =
-            __asyncify_with_deep_sleep::<M, _, _>(ctx, Duration::from_millis(50), async move {
-                let exit_code = process.join().await.unwrap_or_else(|_| Errno::Child.into());
-                tracing::trace!(%exit_code, "triggered child join");
-                JoinStatusResult::ExitNormal(pid, exit_code)
-            })?;
-        return match res {
-            AsyncifyAction::Finish(ctx, result) => ret_result(ctx, result),
-            AsyncifyAction::Unwind => Ok(Errno::Success),
-        };
+        if flags.contains(JoinFlags::NON_BLOCKING) {
+            if let Some(status) = process.try_join() {
+                let exit_code = status.unwrap_or_else(|_| Errno::Child.into());
+                ret_result(ctx, JoinStatusResult::ExitNormal(pid, exit_code))
+            } else {
+                ret_result(ctx, JoinStatusResult::Nothing)
+            }
+        } else {
+            // Wait for the process to finish
+            let process2 = process.clone();
+            let res = __asyncify_with_deep_sleep::<M, _, _>(
+                ctx,
+                Duration::from_millis(50),
+                async move {
+                    let exit_code = process.join().await.unwrap_or_else(|_| Errno::Child.into());
+                    tracing::trace!(%exit_code, "triggered child join");
+                    JoinStatusResult::ExitNormal(pid, exit_code)
+                },
+            )?;
+            match res {
+                AsyncifyAction::Finish(ctx, result) => ret_result(ctx, result),
+                AsyncifyAction::Unwind => Ok(Errno::Success),
+            }
+        }
+    } else {
+        trace!(ret_id = pid.raw(), "status=nothing");
+        ret_result(ctx, JoinStatusResult::Nothing)
     }
-
-    trace!(ret_id = pid.raw(), "status=nothing");
-    ret_result(ctx, JoinStatusResult::Nothing)
 }
